# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""OpenStackClient plugin for Clustering service."""

import logging

from openstack import connection
from openstack import profile
from osc_lib import utils

LOG = logging.getLogger(__name__)

DEFAULT_CLUSTERING_API_VERSION = '1'
API_VERSION_OPTION = 'os_clustering_api_version'
API_NAME = 'clustering'
CURRENT_API_VERSION = '1.8'


def create_connection(prof=None, **kwargs):
    interface = kwargs.pop('interface', None)
    region_name = kwargs.pop('region_name', None)
    user_agent = kwargs.pop('user_agent', None)

    if not prof:
        prof = profile.Profile()
    prof.set_api_version(API_NAME, CURRENT_API_VERSION)

    if interface:
        prof.set_interface(API_NAME, interface)
    if region_name:
        prof.set_region(API_NAME, region_name)

    return connection.Connection(profile=prof, user_agent=user_agent, **kwargs)


def make_client(instance):
    """Returns a clustering proxy"""
    conn = create_connection(
        region_name=instance.region_name,
        interface=instance.interface,
        authenticator=instance.session.auth
    )

    LOG.debug('Connection: %s', conn)
    LOG.debug('Clustering client initialized using OpenStackSDK: %s',
              conn.cluster)
    return conn.cluster


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        '--os-clustering-api-version',
        metavar='<clustering-api-version>',
        default=utils.env(
            'OS_CLUSTERING_API_VERSION',
            default=DEFAULT_CLUSTERING_API_VERSION),
        help='Clustering API version, default=' +
             DEFAULT_CLUSTERING_API_VERSION +
             ' (Env: OS_CLUSTERING_API_VERSION)')
    return parser



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlinclient.tests.functional import base


class ClusterTest(base.OpenStackClientTestBase):
    """Test for clusters"""

    def test_cluster_list(self):
        result = self.openstack('cluster list')
        cluster_list = self.parser.listing(result)
        self.assertTableStruct(cluster_list, ['id', 'name', 'status',
                                              'created_at', 'updated_at'])

    def test_cluster_list_filters(self):
        params = {'name': 'test1', 'status': 'ACTIVE'}
        for k, v in params.items():
            cmd = ('cluster list --filters %s=%s' % (k, v))
            self.openstack(cmd)

    def test_cluster_list_sort(self):
        params = ['name', 'status', 'init_at', 'created_at', 'updated_at']
        for param in params:
            cmd = 'cluster list --sort %s' % param
            self.openstack(cmd)

    def test_cluster_full_id(self):
        self.openstack('cluster list --full-id')

    def test_cluster_limit(self):
        self.openstack('cluster list --limit 1')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlinclient.tests.functional import base


class NodeTest(base.OpenStackClientTestBase):
    """Test for nodes"""

    def test_node_list(self):
        result = self.openstack('cluster node list')
        node_list = self.parser.listing(result)
        self.assertTableStruct(node_list, ['id', 'name', 'index', 'status',
                                           'cluster_id', 'physical_id',
                                           'profile_name', 'created_at',
                                           'updated_at'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Clustering v1 action implementations"""

import logging

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import exceptions as exc
from osc_lib import utils

from senlinclient.common.i18n import _
from senlinclient.common import utils as senlin_utils


class ListAction(command.Lister):
    """List actions."""

    log = logging.getLogger(__name__ + ".ListAction")

    def get_parser(self, prog_name):
        parser = super(ListAction, self).get_parser(prog_name)
        parser.add_argument(
            '--filters',
            metavar='<"key1=value1;key2=value2...">',
            help=_("Filter parameters to apply on returned actions. "
                   "This can be specified multiple times, or once with "
                   "parameters separated by a semicolon. The valid filter "
                   "keys are: ['name', 'target', 'action', 'status']. "
                   "NOTICE: The value of 'target', if provided, "
                   "must be a full ID."),
            action='append'
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            help=_("Sorting option which is a string containing a list of "
                   "keys separated by commas. Each key can be optionally "
                   "appended by a sort direction (:asc or :desc). The valid "
                   "sort keys are: ['name', 'target', 'action', 'created_at',"
                   " 'status']")
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            help=_('Limit the number of actions returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return actions that appear after the given action ID')
        )
        parser.add_argument(
            '--global-project',
            default=False,
            action="store_true",
            help=_('Whether actions from all projects should be listed. '
                   ' Default to False. Setting this to True may demand '
                   'for an admin privilege')
        )
        parser.add_argument(
            '--full-id',
            default=False,
            action="store_true",
            help=_('Print full IDs in list')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        senlin_client = self.app.client_manager.clustering

        columns = ['id', 'name', 'action', 'status', 'target_id', 'depends_on',
                   'depended_by', 'created_at']

        queries = {
            'sort': parsed_args.sort,
            'limit': parsed_args.limit,
            'marker': parsed_args.marker,
            'global_project': parsed_args.global_project,
        }

        if parsed_args.filters:
            queries.update(senlin_utils.format_parameters(parsed_args.filters))

        actions = senlin_client.actions(**queries)

        formatters = {}
        s = None
        if not parsed_args.full_id:
            s = 8
            formatters['id'] = lambda x: x[:s]
            formatters['target_id'] = lambda x: x[:s]

        formatters['depends_on'] = lambda x: '\n'.join(a[:s] for a in x)
        formatters['depended_by'] = lambda x: '\n'.join(a[:s] for a in x)

        return (
            columns,
            (utils.get_item_properties(a, columns,
                                       formatters=formatters)
             for a in actions)
        )


class ShowAction(command.ShowOne):
    """Show detailed info about the specified action."""

    log = logging.getLogger(__name__ + ".ShowAction")

    def get_parser(self, prog_name):
        parser = super(ShowAction, self).get_parser(prog_name)
        parser.add_argument(
            'action',
            metavar='<action>',
            help=_('Name or ID of the action to show the details for')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        senlin_client = self.app.client_manager.clustering
        try:
            action = senlin_client.get_action(parsed_args.action)
        except sdk_exc.ResourceNotFound:
            raise exc.CommandError(_('Action not found: %s')
                                   % parsed_args.action)

        formatters = {
            'inputs': senlin_utils.json_formatter,
            'outputs': senlin_utils.json_formatter,
            'metadata': senlin_utils.json_formatter,
            'data': senlin_utils.json_formatter,
            'depends_on': senlin_utils.list_formatter,
            'depended_by': senlin_utils.list_formatter,
        }
        data = action.to_dict()
        columns = sorted(data.keys())
        return columns, utils.get_dict_properties(data, columns,
                                                  formatters=formatters)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from tempest.lib.cli import base
from tempest.lib.cli import output_parser


class OpenStackClientTestBase(base.ClientTestBase):
    """Command line client base functions."""

    def setUp(self):
        super(OpenStackClientTestBase, self).setUp()
        self.parser = output_parser

    def _get_clients(self):
        cli_dir = os.environ.get(
            'OS_SENLINCLIENT_EXEC_DIR',
            os.path.join(os.path.abspath('.'), '.tox/functional/bin'))

        return base.CLIClient(
            username=os.environ.get('OS_USERNAME'),
            password=os.environ.get('OS_PASSWORD'),
            tenant_name=os.environ.get('OS_TENANT_NAME'),
            uri=os.environ.get('OS_AUTH_URL'),
            cli_dir=cli_dir)

    def openstack(self, *args, **kwargs):
        return self.clients.openstack(*args, **kwargs)
